@@ -22,9 +22,6 @@ incompatibilities from 1.8.x:
 - user_list and user_list_short:
   If a user has both op and voice, only the op flag will be 1.
 
-- dcc_list
-  The address field is always 0.
-
 - add_user_list/sub_user_list/clear_user_list
   These functions do nothing.
 
@@ -45,15 +42,14 @@ incompatibilities from 1.8.x:
 #ifdef ENABLE_NLS
 #include <locale.h>
 #endif
+#ifdef WIN32
+#include <windows.h>
+#endif
 
 #undef PACKAGE
 #include "../../config.h"	/* for #define OLD_PERL */
 #include "xchat-plugin.h"
 
-#ifdef WIN32
-#define strcasecmp stricmp
-#endif
-
 struct perlscript
 {
 	char *name;
@@ -69,6 +65,26 @@ static xchat_plugin *ph; /* plugin handle */
 static int perl_load_file (char *script_name);
 
 
+#ifdef WIN32
+
+static DWORD
+child (char *str)
+{
+	MessageBoxA (0, str, "Perl DLL Error",
+					 MB_OK|MB_ICONHAND|MB_SETFOREGROUND|MB_TASKMODAL);
+	return 0;
+}
+
+static void
+thread_mbox (char *str)
+{
+	DWORD tid;
+
+	CloseHandle (CreateThread (NULL, 0, (LPTHREAD_START_ROUTINE) child,
+					 str, 0, &tid));
+}
+
+#endif
 
 /* leave this before XSUB.h, to avoid readdir() being redefined */
 static void
@@ -258,9 +274,17 @@ static XS (XS_IRC_print_with_channel)
 	int junk;
 	dXSARGS;
 	void *ctx, *old_ctx;
+	char *server = NULL;
+
+	if (items > 2)
+	{
+		server = SvPV (ST (2), junk);
+		if (!server[0])
+			server = NULL;
+	}
 
 	old_ctx = xchat_get_context (ph);
-	ctx = xchat_find_context (ph, SvPV (ST (1), junk), SvPV (ST (2), junk));
+	ctx = xchat_find_context (ph, server, SvPV (ST (1), junk));
 	if (ctx)
 	{
 		xchat_set_context (ph, ctx);
@@ -735,9 +759,8 @@ static XS (XS_IRC_dcc_list)
 			else
 				XST_mIV (i, xchat_list_int (ph, list, "resume"));
 			i++;
-			XST_mIV (i, 0);	/* FIXME: addr */
+			XST_mIV (i, xchat_list_int (ph, list, "address32"));
 			i++;
-
 			file = xchat_list_str (ph, list, "destfile");
 			if (!file)
 				file = "";
@@ -758,9 +781,13 @@ static XS (XS_IRC_command)
 {
 	int junk;
 	dXSARGS;
+	char *cmd = SvPV (ST (0), junk);
 
-	/* skip the forward slash */
-	xchat_command (ph, SvPV (ST (0), junk) + 1);
+	if (cmd[0] == '/')
+		/* skip the forward slash */
+		xchat_command (ph, cmd + 1);
+	else
+		xchat_commandf (ph, "say %s", cmd);
 
 	XSRETURN_EMPTY;
 }
@@ -771,13 +798,20 @@ static XS (XS_IRC_command_with_server)
 	int junk;
 	dXSARGS;
 	void *ctx, *old_ctx;
+	char *cmd = SvPV (ST (0), junk);
 
 	old_ctx = xchat_get_context (ph);
 	ctx = xchat_find_context (ph, SvPV (ST (1), junk), NULL);
 	if (ctx)
 	{
 		xchat_set_context (ph, ctx);
-		xchat_command (ph, SvPV (ST (0), junk) + 1);
+
+		if (cmd[0] == '/')
+			/* skip the forward slash */
+			xchat_command (ph, cmd + 1);
+		else
+			xchat_commandf (ph, "say %s", cmd);
+
 		xchat_set_context (ph, old_ctx);
 		XSRETURN_YES;
 	}
@@ -985,6 +1019,27 @@ perl_init (void)
 static int
 perl_load_file (char *script_name)
 {
+#ifdef WIN32
+	static int have_lib = FALSE;
+	HINSTANCE lib;
+
+	if (!have_lib)
+	{
+		lib = LoadLibrary (PERLDLL);
+		if (!lib)
+		{
+			thread_mbox ("Cannot open " PERLDLL "\n\n"
+							 "You must have ActivePerl installed in order to\n"
+							 "run perl scripts.\n\n"
+							 "http://www.activestate.com/ActivePerl/\n\n"
+							 "Make sure perl's bin directory is in your PATH.");
+			return FALSE;
+		}
+		have_lib = TRUE;
+		FreeLibrary (lib);
+	}
+#endif
+
 	if (my_perl == NULL)
 		perl_init ();
 
@@ -1055,18 +1110,31 @@ perl_command_unloadall (char *word[], char *word_eol[], void *userdata)
 static int
 perl_command_unload (char *word[], char *word_eol[], void *userdata)
 {
-#if 0
 	int len;
+	struct perlscript *scp;
+	GSList *list;
 
+	/* try it by filename */
 	len = strlen (word[2]);
 	if (len > 3 && strcasecmp (".pl", word[2] + len - 3) == 0)
 	{
-
 		/* if only unloading was possible with this shitty interface :( */
-
+		xchat_print (ph, "Unloading individual perl scripts is not supported.\n");
 		return XCHAT_EAT_XCHAT;
 	}
-#endif
+
+	/* try it by script name */
+	list = perl_list;
+	while (list)
+	{
+		scp = list->data;
+		if (strcasecmp (scp->name, word[2]) == 0)
+		{
+			xchat_print (ph, "Unloading individual perl scripts is not supported.\n");
+			return XCHAT_EAT_XCHAT;
+		}
+		list = list->next;
+	}
 
 	return XCHAT_EAT_NONE;
 }
@@ -1096,12 +1164,26 @@ perl_command_load (char *word[], char *word_eol[], void *userdata)
 	return XCHAT_EAT_NONE;
 }
 
+
+/* Reinit safeguard */
+
+static int initialized = 0;
+static int reinit_tried = 0;
+
 int
 xchat_plugin_init (xchat_plugin *plugin_handle,
 				char **plugin_name, char **plugin_desc, char **plugin_version,
 				char *arg)
 {
 	ph = plugin_handle;
+	
+	if (initialized != 0)
+	{
+		xchat_print (ph, "Perl interface already loaded\n");
+		reinit_tried++;
+		return 0;
+	}
+	initialized = 1;
 
 	*plugin_name = "Perl";
 	*plugin_version = VERSION;
@@ -1113,13 +1195,23 @@ xchat_plugin_init (xchat_plugin *plugin_handle,
 
 	perl_auto_load ();
 
+	xchat_print (ph, "Perl interface loaded\n");
+
 	return 1;
 }
 
 int
 xchat_plugin_deinit (xchat_plugin *plugin_handle)
 {
+	if (reinit_tried)
+	{
+		reinit_tried--;
+		return 1;
+	}
+
 	perl_end ();
 
+	xchat_print (plugin_handle, "Perl interface unloaded\n");
+
 	return 1;
 }
