@@ -190,25 +190,29 @@ cfg_get_str (char *cfg, char *var, char *dest)
 	}
 }
 
-static void
+static int
 cfg_put_str (int fh, char *var, char *value)
 {
 	char buf[512];
+	int len;
 
 	snprintf (buf, sizeof buf, "%s = %s\n", var, value);
-	write (fh, buf, strlen (buf));
+	len = strlen (buf);
+	return (write (fh, buf, len) == len);
 }
 
-void
+int
 cfg_put_int (int fh, int value, char *var)
 {
 	char buf[400];
+	int len;
 
 	if (value == -1)
 		value = 1;
 
 	snprintf (buf, sizeof buf, "%s = %d\n", var, value);
-	write (fh, buf, strlen (buf));
+	len = strlen (buf);
+	return (write (fh, buf, len) == len);
 }
 
 int
@@ -239,21 +243,64 @@ cfg_get_int (char *cfg, char *var)
 
 char *xdir = 0;
 
+#ifdef WIN32
+
+#include <windows.h>
+
+static gboolean
+get_reg_str (const char *sub, const char *name, char *out, DWORD len)
+{
+	HKEY hKey;
+	DWORD t;
+
+	if (RegOpenKeyEx (HKEY_CURRENT_USER, sub, 0, KEY_READ, &hKey) ==
+			ERROR_SUCCESS)
+	{
+		if (RegQueryValueEx (hKey, name, NULL, &t, out, &len) != ERROR_SUCCESS ||
+			 t != REG_SZ)
+		{
+			RegCloseKey (hKey);
+			return FALSE;
+		}
+		out[len-1] = 0;
+		RegCloseKey (hKey);
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+char *
+get_xdir (void)
+{
+	if (!xdir)
+	{
+		char out[256];
+
+		if (!get_reg_str ("Software\\Microsoft\\Windows\\CurrentVersion\\"
+				"Explorer\\Shell Folders", "AppData", out, sizeof (out)))
+			return "./config";
+		xdir = malloc (strlen (out) + 10);
+		sprintf (xdir, "%s\\X-Chat 2", out);
+	}
+	return xdir;
+}
+
+#else
+
 char *
 get_xdir (void)
 {
-#ifndef WIN32
 	if (!xdir)
 	{
 		xdir = malloc (strlen (g_get_home_dir ()) + 9);
 		sprintf (xdir, "%s/.xchat2", g_get_home_dir ());
 	}
 	return xdir;
-#else
-	return "./config";
-#endif
 }
 
+#endif	/* !WIN32 */
+
 static void
 check_prefs_dir (void)
 {
@@ -362,6 +409,7 @@ const struct prefs vars[] = {
 
 	{"irc_auto_rejoin", P_OFFINT (autorejoin), TYPE_BOOL},
 	{"irc_ban_type", P_OFFINT (bantype), TYPE_INT},
+	{"irc_conf_mode", P_OFFINT (confmode), TYPE_BOOL},
 	{"irc_extra_hilight", P_OFFSET (bluestring), TYPE_STR},
 	{"irc_invisible", P_OFFINT (invisible), TYPE_BOOL},
 	{"irc_logging", P_OFFINT (logging), TYPE_BOOL},
@@ -371,7 +419,6 @@ const struct prefs vars[] = {
 	{"irc_nick2", P_OFFSET (nick2), TYPE_STR},
 	{"irc_nick3", P_OFFSET (nick3), TYPE_STR},
 	{"irc_part_reason", P_OFFSET (partreason), TYPE_STR},
-	{"irc_persist_chans", P_OFFINT (persist_chans), TYPE_BOOL},
 	{"irc_raw_modes", P_OFFINT (raw_modes), TYPE_BOOL},
 	{"irc_real_name", P_OFFSET (realname), TYPE_STR},
 	{"irc_quit_reason", P_OFFSET (quitreason), TYPE_STR},
@@ -414,7 +461,7 @@ const struct prefs vars[] = {
 
 	{"tab_dialogs", P_OFFINT (privmsgtab), TYPE_BOOL},
 	{"tab_chans", P_OFFINT (tabchannels), TYPE_BOOL},
-	{"tab_limited_hilight", P_OFFINT (limitedtabhighlight), TYPE_BOOL},
+/*	{"tab_limited_hilight", P_OFFINT (limitedtabhighlight), TYPE_BOOL},*/
 	{"tab_new_to_front", P_OFFINT (newtabstofront), TYPE_BOOL},
 	{"tab_notices", P_OFFINT (notices_tabs), TYPE_BOOL},
 	{"tab_position", P_OFFINT (tabs_position), TYPE_INT},
@@ -446,7 +493,7 @@ load_config (void)
 {
 	struct stat st;
 	char *cfg;
-	const char *username;
+	const char *username, *realname;
 	int res, val, i, fh;
 
 	check_prefs_dir ();
@@ -454,6 +501,10 @@ load_config (void)
 	if (!username)
 		username = "root";
 
+	realname = g_get_real_name ();
+	if ((realname && realname[0] == 0) || !realname)
+		realname = username;
+
 	memset (&prefs, 0, sizeof (struct xchatprefs));
 
 	/* put in default values, anything left out is automatically zero */
@@ -478,7 +529,7 @@ load_config (void)
 	prefs.privmsgtab = 1;
 	prefs.nickcompletion = 1;
 	prefs.style_inputbox = 1;
-	prefs.slist_select = 10;
+	prefs.slist_select = 9;
 	prefs.nu_color = 4;
 	prefs.dccpermissions = 0600;
 	prefs.max_lines = 300;
@@ -508,7 +559,6 @@ load_config (void)
 	prefs.chanmodebuttons = 1;
 	prefs.userhost = 1;
 	prefs.userlistbuttons = 1;
-	prefs.persist_chans = 1;
 	prefs.perc_color = 1;
 	prefs.dcc_send_fillspaces = 1;
 	prefs.mainwindow_save = 1;
@@ -522,16 +572,17 @@ load_config (void)
 	strcat (prefs.nick2, "_");
 	strcpy (prefs.nick3, username);
 	strcat (prefs.nick3, "__");
-	strcpy (prefs.realname, username);
+	strcpy (prefs.realname, realname);
 	strcpy (prefs.username, username);
 #ifdef WIN32
 	strcpy (prefs.sounddir, "./sound");
-	strcpy (prefs.dccdir, "./download");
-	/*strcpy (prefs.dcc_completed_dir, "./dcc");*/
+	if (strcmp (get_xdir (), "./config") != 0)
+		sprintf (prefs.dccdir, "%s\\download", get_xdir ());
+	else
+		strcpy (prefs.dccdir, "./download");
 #else
 	sprintf (prefs.sounddir, "%s/sound", g_get_home_dir ());
 	sprintf (prefs.dccdir, "%s/dcc", g_get_home_dir ());
-	/*strcpy (prefs.dcc_completed_dir, prefs.dccdir);*/
 #endif
 	strcpy (prefs.doubleclickuser, "QUOTE WHOIS %s");
 	strcpy (prefs.awayreason, _("I'm busy"));
@@ -602,31 +653,67 @@ int
 save_config (void)
 {
 	int fh, i;
+	char *new_config, *config;
 
 	check_prefs_dir ();
 
-	fh = open (default_file (), OFLAGS | O_TRUNC | O_WRONLY | O_CREAT, 0600);
+	config = default_file ();
+	new_config = malloc (strlen (config) + 5);
+	strcpy (new_config, config);
+	strcat (new_config, ".new");
+	
+	fh = open (new_config, OFLAGS | O_TRUNC | O_WRONLY | O_CREAT, 0600);
 	if (fh == -1)
+	{
+		free (new_config);
 		return 0;
+	}
 
-	cfg_put_str (fh, "version", VERSION);
+	if (!cfg_put_str (fh, "version", VERSION))
+	{
+		free (new_config);
+		return 0;
+	}
+		
 	i = 0;
 	do
 	{
 		switch (vars[i].type)
 		{
 		case TYPE_STR:
-			cfg_put_str (fh, vars[i].name, (char *) &prefs + vars[i].offset);
+			if (!cfg_put_str (fh, vars[i].name, (char *) &prefs + vars[i].offset))
+			{
+				free (new_config);
+				return 0;
+			}
 			break;
 		case TYPE_INT:
 		case TYPE_BOOL:
-			cfg_put_int (fh, *((int *) &prefs + vars[i].offset), vars[i].name);
+			if (!cfg_put_int (fh, *((int *) &prefs + vars[i].offset), vars[i].name))
+			{
+				free (new_config);
+				return 0;
+			}
 		}
 		i++;
 	}
 	while (vars[i].name);
 
-	close (fh);
+	if (close (fh) == -1)
+	{
+		free (new_config);
+		return 0;
+	}
+
+#ifdef WIN32
+	unlink (config);	/* win32 can't rename to an existing file */
+#endif
+	if (rename (new_config, config) == -1)
+	{
+		free (new_config);
+		return 0;
+	}
+	free (new_config);
 
 	return 1;
 }
