@@ -218,7 +218,7 @@ dcc_check_timeouts (void)
 						EMIT_SIGNAL (XP_TE_DCCSTALL, dcc->serv->front_session,
 										 dcctypes[(int) dcc->type],
 										 file_part (dcc->file), dcc->nick, NULL, 0);
-						dcc_close (dcc, 0, TRUE);
+						dcc_close (dcc, STAT_ABORTED, FALSE);
 					}
 				}
 			}
@@ -233,7 +233,7 @@ dcc_check_timeouts (void)
 						EMIT_SIGNAL (XP_TE_DCCTOUT, dcc->serv->front_session,
 										 dcctypes[(int) dcc->type],
 										 file_part (dcc->file), dcc->nick, NULL, 0);
-						dcc_close (dcc, 0, TRUE);
+						dcc_close (dcc, STAT_ABORTED, FALSE);
 					}
 				}
 			}
@@ -574,7 +574,8 @@ dcc_read_chat (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 			}
 			sprintf (tbuf, "%d", dcc->port);
 			EMIT_SIGNAL (XP_TE_DCCCHATF, dcc->serv->front_session, dcc->nick,
-							 net_ip (dcc->addr), tbuf, NULL, 0);
+							 net_ip (dcc->addr), tbuf,
+							 errorstring ((len < 0) ? sock_error () : 0), 0);
 			dcc_close (dcc, STAT_FAILED, FALSE);
 			return TRUE;
 		}
@@ -701,7 +702,8 @@ dcc_read (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 			}
 failedabort:
 			EMIT_SIGNAL (XP_TE_DCCRECVERR, dcc->serv->front_session, dcc->file,
-							 dcc->destfile, dcc->nick, NULL, 0);
+							 dcc->destfile, dcc->nick,
+							 errorstring ((n < 0) ? sock_error () : 0), 0);
 			dcc_close (dcc, STAT_FAILED, FALSE);
 			return TRUE;
 		}
@@ -741,12 +743,29 @@ dcc_connect_finished (GIOChannel *source, GIOCondition condition, struct DCC *dc
 		dcc->iotag = 0;
 	}
 
+#ifdef WIN32
+	if (condition & G_IO_ERR)
+	{
+		int len;
+
+		/* find the last errno for this socket */
+		len = sizeof (er);
+		getsockopt (dcc->sok, SOL_SOCKET, SO_ERROR, (char *)&er, &len);
+		EMIT_SIGNAL (XP_TE_DCCCONFAIL, dcc->serv->front_session,
+						 dcctypes[(int) dcc->type], dcc->nick, errorstring (er),
+						 NULL, 0);
+		dcc->dccstat = STAT_FAILED;
+		update_dcc_window (dcc->type);
+		return TRUE;
+	}
+
+#else
 	memset (&addr, 0, sizeof (addr));
 	addr.sin_port = htons (dcc->port);
 	addr.sin_family = AF_INET;
 	addr.sin_addr.s_addr = htonl (dcc->addr);
 
-	/* check if it's already connected */
+	/* check if it's already connected; This always fails on winXP */
 	if (connect (dcc->sok, (struct sockaddr *) &addr, sizeof (addr)) != 0)
 	{
 		er = sock_error ();
@@ -764,6 +783,7 @@ dcc_connect_finished (GIOChannel *source, GIOCondition condition, struct DCC *dc
 			return TRUE;
 		}
 	}
+#endif
 
 	dcc->dccstat = STAT_ACTIVE;
 	snprintf (host, sizeof host, "%s:%d", net_ip (dcc->addr), dcc->port);
@@ -884,7 +904,8 @@ dcc_send_data (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 abortit:
 		free (buf);
 		EMIT_SIGNAL (XP_TE_DCCSENDFAIL, dcc->serv->front_session,
-						 file_part (dcc->file), dcc->nick, NULL, NULL, 0);
+						 file_part (dcc->file), dcc->nick,
+						 errorstring (sock_error ()), NULL, 0);
 		dcc_close (dcc, STAT_FAILED, FALSE);
 		return TRUE;
 	}
@@ -928,7 +949,8 @@ dcc_read_ack (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 				return TRUE;
 		}
 		EMIT_SIGNAL (XP_TE_DCCSENDFAIL, dcc->serv->front_session,
-						 file_part (dcc->file), dcc->nick, NULL, NULL, 0);
+						 file_part (dcc->file), dcc->nick,
+						 errorstring ((len < 0) ? sock_error () : 0), NULL, 0);
 		dcc_close (dcc, STAT_FAILED, FALSE);
 		return TRUE;
 	}
@@ -1632,8 +1654,9 @@ dontresume:
 				fe_dcc_update_recv_win ();
 		}
 		sprintf (outbuf, "%lu", size);
+		snprintf (outbuf + 24, 300, "%s:%d", net_ip (dcc->addr), dcc->port);
 		EMIT_SIGNAL (XP_TE_DCCSENDOFFER, sess->server->front_session, nick,
-						 file, outbuf, NULL, 0);
+						 file, outbuf, outbuf + 24, 0);
 	} else
 		EMIT_SIGNAL (XP_TE_DCCGENERICOFFER, sess->server->front_session,
 						 word_eol[4] + 2, nick, NULL, NULL, 0);
