@@ -14,6 +14,8 @@
 #define TYPE_CHATRECV 2
 #define TYPE_CHATSEND 3
 
+#define CPS_AVG_WINDOW 10
+
 struct DCC
 {
 	struct server *serv;
@@ -26,6 +28,11 @@ struct DCC
 	int port;
 	int pasvid;						/* mIRC's passive DCC id */
 	int cps;
+
+	GTimeVal lastcpstv, firstcpstv;
+	int lastcpspos;
+	int maxcps;
+
 	unsigned int size;
 	unsigned int resumable;
 	unsigned int ack;
@@ -33,14 +40,18 @@ struct DCC
 	time_t starttime;
 	time_t offertime;
 	time_t lasttime;
-	char *file;
-	char *destfile;
+	char *file;					/* utf8 */
+	char *destfile;			/* utf8 */
+	char *destfile_fs;		/* local filesystem encoding */
 	char *nick;
 	char type;				  /* 0 = SEND  1 = RECV  2 = CHAT */
 	char dccstat;			  /* 0 = QUEUED  1 = ACTIVE  2 = FAILED  3 = DONE */
 	unsigned int fastsend:1;
 	unsigned int ackoffset:1;	/* is reciever sending acks as an offset from */
-};										/* the resume point? */
+										/* the resume point? */
+	unsigned int throttled:2;	/* 0x1 = per send/get throttle
+											0x2 = global throttle */
+};
 
 struct dcc_chat
 {
@@ -63,7 +74,7 @@ void dcc_check_timeouts (void);
 void dcc_change_nick (server *serv, char *oldnick, char *newnick);
 void dcc_notify_kill (struct server *serv);
 struct DCC *dcc_write_chat (char *nick, char *text);
-void dcc_send (struct session *sess, char *tbuf, char *to, char *file);
+void dcc_send (struct session *sess, char *tbuf, char *to, char *file, int maxcps);
 struct DCC *find_dcc (char *nick, char *file, int type);
 void dcc_get_nick (struct session *sess, char *nick);
 void dcc_chat (session *sess, char *nick);
