@@ -63,8 +63,7 @@ color_of (char *name)
 void
 clear_channel (session *sess)
 {
-	if (prefs.persist_chans)
-		strcpy (sess->waitchannel, sess->channel);
+	strcpy (sess->waitchannel, sess->channel);
 	sess->channel[0] = 0;
 
 	log_close (sess);
@@ -248,7 +247,11 @@ inbound_action (session *sess, char *tbuf, char *chan, char *from, char *text,
 	if (!sess)
 		sess = def;
 
-	sess->highlight_tab = TRUE;
+	if (sess != current_tab)
+	{
+		sess->msg_said = TRUE;
+		sess->new_data = FALSE;
+	}
 
 	if (!fromme)
 	{
@@ -287,7 +290,6 @@ inbound_chanmsg (server *serv, char *tbuf, char *chan, char *from, char *text,
 	if (!sess)
 		return;
 
-	sess->highlight_tab = TRUE;
 	if (sess != current_tab)
 	{
 		sess->msg_said = TRUE;
@@ -389,7 +391,7 @@ find_unused_session (server *serv)
 		if (sess->type == SESS_CHANNEL && sess->channel[0] == 0 &&
 			 sess->server == serv)
 		{
-			if (!prefs.persist_chans || sess->waitchannel[0] == 0)
+			if (sess->waitchannel[0] == 0)
 				return sess;
 		}
 		list = list->next;
@@ -743,6 +745,12 @@ inbound_notice (server *serv, char *outbuf, char *to, char *nick, char *msg,
 		sess = find_channel (serv, ptr);
 	}
 
+	if (!sess && ptr[0] == '+')
+	{
+		ptr++;
+		sess = find_channel (serv, ptr);
+	}
+
 	if (strcmp (nick, ip) == 0)
 		server_notice = TRUE;
 
@@ -880,6 +888,9 @@ check_willjoin_channels (server *serv)
 				po = strchr (sess->waitchannel, ',');
 				if (po)
 					*po = 0;
+				po = strchr (sess->waitchannel, ' ');
+				if (po)
+					*po = 0;
 			}
 		}
 		list = list->next;
