@@ -48,6 +48,7 @@
 #include "inbound.h"
 #include "text.h"
 #include "xchatc.h"
+#include "servlist.h"
 #include "server.h"
 #include "outbound.h"
 
@@ -157,6 +158,8 @@ process_data_init (unsigned char *buf, unsigned char *cmd, char *word[],
 	int j = 0;
 	int len;
 
+	word[0] = "\000\000";
+	word_eol[0] = "\000\000";
 	word[1] = buf;
 	word_eol[1] = cmd;
 
@@ -496,6 +499,34 @@ cmd_unban (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	}
 }
 
+static int
+cmd_charset (struct session *sess, char *tbuf, char *word[], char *word_eol[])
+{
+	server *serv = sess->server;
+	const char *locale = NULL;
+
+	if (!word[2][0])
+	{
+		g_get_charset (&locale);
+		PrintTextf (sess, "Current charset: %s\n",
+						serv->encoding ? serv->encoding : locale);
+		return TRUE;
+	}
+
+	if (servlist_check_encoding (word[2]))
+	{
+		if (serv->encoding)
+			free (serv->encoding);
+		serv->encoding = strdup (word[2]);
+		PrintTextf (sess, "Charset changed to: %s\n", word[2]);
+	} else
+	{
+		PrintTextf (sess, "\0034Unknown charset:\017 %s\n", word[2]);
+	}
+
+	return TRUE;
+}
+
 static int
 cmd_clear (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
@@ -547,21 +578,24 @@ cmd_close (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 static int
 cmd_ctcp (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
+	int mbl;
 	char *to = word[2];
 	if (*to)
 	{
 		char *msg = word_eol[3];
 		if (*msg)
 		{
-			char *cmd = msg;
+			unsigned char *cmd = msg;
 
+			/* make the first word upper case (as per RFC) */
 			while (1)
 			{
-				/* FIXME: utf8 */
 				if (*cmd == ' ' || *cmd == 0)
 					break;
-				*cmd = toupper (*cmd);
-				cmd++;
+				mbl = g_utf8_skip[*cmd];
+				if (mbl == 1)
+					*cmd = toupper (*cmd);
+				cmd += mbl;
 			}
 
 			sess->server->p_ctcp (sess->server, to, msg);
@@ -1658,7 +1692,7 @@ cmd_join (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	{
 		char *po, *pass = word[3];
 		sess->server->p_join (sess->server, chan, pass);
-		if (sess->channel[0] == 0 && !prefs.persist_chans)
+		if (sess->channel[0] == 0 && sess->waitchannel[0])
 		{
 			po = strchr (chan, ',');
 			if (po)
@@ -1807,7 +1841,7 @@ cmd_load (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	}
 #endif
 
-	sprintf (tbuf, "Unknown file type %s\n", word[2]);
+	sprintf (tbuf, "Unknown file type %s. Maybe you need to install the Perl or Python plugin?\n", word[2]);
 	PrintText (sess, tbuf);
 
 	return FALSE;
@@ -2454,6 +2488,7 @@ const struct commands xc_cmds[] = {
 	 N_("BAN <mask> [<bantype>], bans everyone matching the mask from the current channel. If they are already on the channel this doesn't kick them (needs chanop)")},
 	{"UNBAN", cmd_unban, 1, 1,
 	 N_("UNBAN <mask> [<mask>...], unbans the specified masks.")},
+	{"CHARSET", cmd_charset, 0, 0, 0},
 	{"CLEAR", cmd_clear, 0, 0, N_("CLEAR, Clears the current text window")},
 	{"CLOSE", cmd_close, 0, 0, N_("CLOSE, Closes the current window/tab")},
 
@@ -2646,6 +2681,7 @@ auto_insert (char *dest, unsigned char *src, char *word[], char *word_eol[],
 	struct tm *tm_ptr;
 	char *utf;
 	int utf_len;
+	char *orig = dest;
 
 	while (src[0])
 	{
@@ -2676,10 +2712,20 @@ auto_insert (char *dest, unsigned char *src, char *word[], char *word_eol[],
 						num = src[0] - '0';	/* ascii to decimal */
 						if (*word[num] == 0)
 							return 0;
+
 						if (src[-1] == '%')
-							strcpy (dest, word[num]);
+							utf = word[num];
 						else
-							strcpy (dest, word_eol[num]);
+							utf = word_eol[num];
+
+						/* avoid recusive usercommand overflow */
+						if ((dest - orig) + strlen (utf) >= 2048)
+						{
+							PrintText (0, "UserCommand buffer overflow.\n");
+							return 0;
+						}
+
+						strcpy (dest, utf);
 						dest += strlen (dest);
 					}
 				}
@@ -3109,6 +3155,24 @@ handle_user_input (session *sess, char *text, int history, int nocommand)
 		return 1;
 	}
 
+	if (prefs.cmdchar[0] == '/')
+	{
+		int i;
+		const char *unix_dirs [] = {
+			"/bin/", "/boot/", "/dev/",
+			"/etc/", "/home/", "/lib/",
+			"/lost+found/", "/mnt/", "/opt/",
+			"/proc/", "/root/", "/sbin/",
+			"/tmp/", "/usr/", "/var/",
+			"/gnome/", NULL};
+		for (i = 0; unix_dirs[i] != NULL; i++)
+			if (strncmp (text, unix_dirs[i], strlen (unix_dirs[i]))==0)
+			{
+				handle_say (sess, text, TRUE);
+				return 1;
+			}
+	}
+
 	return handle_command (sess, text + 1, TRUE);
 }
 
