@@ -191,6 +191,11 @@ plugin_list_add (xchat_context *ctx, char *filename, char *name, char *desc,
 	return pl;
 }
 
+static void
+xchat_dummy (xchat_plugin *ph)
+{
+}
+
 /* Load a static plugin */
 
 void
@@ -209,7 +214,6 @@ plugin_add (session *sess, char *filename, void *handle, void *init_func,
 
 	if (!fake)
 	{
-#ifdef WIN32
 		pl->xchat_hook_command = xchat_hook_command;
 		pl->xchat_hook_server = xchat_hook_server;
 		pl->xchat_hook_print = xchat_hook_print;
@@ -234,7 +238,16 @@ plugin_add (session *sess, char *filename, void *handle, void *init_func,
 		pl->xchat_list_int = xchat_list_int;
 		pl->xchat_plugingui_add = xchat_plugingui_add;
 		pl->xchat_plugingui_remove = xchat_plugingui_remove;
-#endif
+		pl->xchat_emit_print = xchat_emit_print;
+		/* incase new plugins are loaded on older xchat */
+		pl->xchat_dummy8 = xchat_dummy;
+		pl->xchat_dummy7 = xchat_dummy;
+		pl->xchat_dummy6 = xchat_dummy;
+		pl->xchat_dummy5 = xchat_dummy;
+		pl->xchat_dummy4 = xchat_dummy;
+		pl->xchat_dummy3 = xchat_dummy;
+		pl->xchat_dummy2 = xchat_dummy;
+		pl->xchat_dummy1 = xchat_dummy;
 
 		/* run xchat_plugin_init, if it returns 0, close the plugin */
 		if (((xchat_init_func *)init_func) (pl, &pl->name, &pl->desc, &pl->version, arg) == 0)
@@ -368,6 +381,7 @@ plugin_auto_load (session *sess)
 	ps = sess;
 #ifdef WIN32
 	for_files ("./plugins", "*.dll", plugin_auto_load_cb);
+	for_files (get_xdir (), "*.dll", plugin_auto_load_cb);
 #else
 	for_files (XCHATLIBDIR"/plugins", "*.so", plugin_auto_load_cb);
 	for_files (get_xdir (), "*.so", plugin_auto_load_cb);
@@ -781,6 +795,9 @@ xchat_find_context (xchat_plugin *ph, char *servname, char *channel)
 	server *serv;
 	session *sess;
 
+	if (servname == NULL && channel == NULL)
+		return current_sess;
+
 	slist = serv_list;
 	while (slist)
 	{
@@ -958,8 +975,8 @@ xchat_list_fields (xchat_plugin *ph, const char *name)
 {
 	static const char *dcc_fields[] =
 	{
-		"icps",		"sdestfile","sfile",		"snick",		"ipos",
-		"iresume",	"isize",		"istatus", 	"itype",		NULL
+		"iaddress32","icps",		"sdestfile","sfile",		"snick",	"iport",
+		"ipos",		"iresume",	"isize",		"istatus", 	"itype",		NULL
 	};
 	static const char *channels_fields[] =
 	{
@@ -1062,8 +1079,12 @@ xchat_list_int (xchat_plugin *ph, xchat_list *xlist, const char *name)
 	case LIST_DCC:
 		switch (hash)
 		{
+		case 0x34207553: /* address32 */
+			return ((struct DCC *)data)->addr;
 		case 0x181a6: /* cps */
 			return ((struct DCC *)data)->cps;
+		case 0x349881: /* port */
+			return ((struct DCC *)data)->port;
 		case 0x1b254: /* pos */
 			return ((struct DCC *)data)->pos;
 		case 0xc84dc82d: /* resume */
@@ -1109,3 +1130,29 @@ xchat_plugingui_remove (xchat_plugin *ph, void *handle)
 	plugin_free (handle, FALSE, FALSE);
 #endif
 }
+
+int
+xchat_emit_print (xchat_plugin *ph, char *event_name, ...)
+{
+	va_list args;
+	char *argv[4];
+	int i = 0;
+
+	memset (&argv, 0, sizeof (argv));
+	va_start (args, event_name);
+	while (1)
+	{
+		argv[i] = va_arg (args, char *);
+		if (!argv[i])
+			break;
+		i++;
+		if (i >= 4)
+			break;
+	}
+
+	i = text_emit_by_name (event_name, ph->context, argv[0], argv[1],
+								  argv[2], argv[3]);
+	va_end (args);
+
+	return i;
+}
