@@ -616,7 +616,7 @@ ssl_do_connect (server * serv)
 		return (0);					  /* remove it (0) */
 	} else
 	{
-		if (serv->ssl->session->time + SSLTMOUT < time (NULL))
+		if (serv->ssl->session && serv->ssl->session->time + SSLTMOUT < time (NULL))
 		{
 			snprintf (buf, sizeof (buf), "SSL handshake timed out");
 			EMIT_SIGNAL (XP_TE_CONNFAIL, serv->server_session, buf, NULL,
@@ -1251,7 +1251,7 @@ server_connect (server *serv, char *hostname, int port, int no_login)
 	{
 		/* use default port for this server type */
 		port = 6667;
-#ifdef USE_SSL
+#ifdef USE_OPENSSL
 		if (serv->use_ssl)
 			port = 9999;
 #endif
