@@ -261,28 +261,34 @@ static void
 log_create_filename (char *buf, char *servname, char *channame, char *netname)
 {
 	char fn[256];
-	char *dir, *sep;
-	int pathlen=510, c=0;
+	unsigned char *tmp, *dir, *sep;
+	int pathlen=510, c=0, mbl;
 
 	if (!rfc_casecmp (channame, servname))
 	{
-		channame = strdup ("server");
+		channame = g_strdup ("server");
 	} else
 	{
-		channame = strdup (channame);
-		tolowerStr (channame);
-		/* win32 can't handle filenames with the '|' character */
-		dir = channame;
-		while (*dir)
+		sep = tmp = strdup (channame);
+		while (*tmp)
 		{
-#ifdef WIN32
-			if (*dir == '|')
+			mbl = g_utf8_skip[*tmp];
+			if (mbl == 1)
+			{
+#ifndef WIN32
+				*tmp = tolower (*tmp);
+				if (*tmp == '/')
 #else
-			if (*dir == '/')
+				/* win32 can't handle filenames with the '|' character */
+				if (*tmp == '|')
 #endif
-				*dir = '_';
-			dir++;
+					*tmp = '_';
+			}
+			tmp += mbl;
 		}
+
+		channame = g_filename_from_utf8 (sep, -1, 0, 0, 0);
+		free (sep);
 	}
 
 	snprintf (buf, 512, "%s/xchatlogs", get_xdir ());
@@ -293,7 +299,7 @@ log_create_filename (char *buf, char *servname, char *channame, char *netname)
 		mkdir (buf, S_IRUSR | S_IWUSR | S_IXUSR);
 #endif
 	auto_insert (fn, prefs.logmask, NULL, NULL, "", channame, "", "", netname, servname);
-	free (channame);
+	g_free (channame);
 
 	snprintf (buf, 512, "%s/xchatlogs/%s", get_xdir (), fn);
 
@@ -353,9 +359,9 @@ log_open_file (char *servname, char *channame, char *netname)
 
 	log_create_filename (buf, servname, channame, netname);
 #ifdef WIN32
-	fd = open (buf, O_CREAT | O_APPEND | O_WRONLY | OFLAGS, S_IREAD|S_IWRITE);
+	fd = open (buf, O_CREAT | O_APPEND | O_WRONLY, S_IREAD|S_IWRITE);
 #else
-	fd = open (buf, O_CREAT | O_APPEND | O_WRONLY | OFLAGS, 0644);
+	fd = open (buf, O_CREAT | O_APPEND | O_WRONLY, 0644);
 #endif
 	if (fd == -1)
 		return -1;
@@ -854,7 +860,8 @@ static char *pevt_kill_help[] = {
 static char *pevt_dccchaterr_help[] = {
 	N_("Nickname"),
 	N_("IP address"),
-	N_("port"),
+	N_("Port"),
+	N_("Error name"),
 };
 
 static char *pevt_dccstall_help[] = {
@@ -871,6 +878,7 @@ static char *pevt_dccrecverr_help[] = {
 	N_("Filename"),
 	N_("Destination filename"),
 	N_("Nickname"),
+	N_("Error name"),
 };
 
 static char *pevt_dccrecvcomp_help[] = {
@@ -900,6 +908,7 @@ static char *pevt_dcccon_help[] = {
 static char *pevt_dccsendfail_help[] = {
 	N_("Filename"),
 	N_("Nickname"),
+	N_("Error name"),
 };
 
 static char *pevt_dccsendcomp_help[] = {
@@ -933,6 +942,7 @@ static char *pevt_dccsendoffer_help[] = {
 	N_("Nickname"),
 	N_("Filename"),
 	N_("Size"),
+	N_("IP address"),
 };
 
 static char *pevt_dccgenericoffer_help[] = {
@@ -1492,6 +1502,31 @@ text_emit (int index, session *sess, char *a, char *b, char *c, char *d)
 	display_event (pntevts[index], sess, te[index].num_args, word);
 }
 
+int
+text_emit_by_name (char *name, session *sess, char *a, char *b, char *c, char *d)
+{
+	int i = 0;
+
+	i = pevent_find (name, &i);
+	if (i >= 0)
+	{
+		/* incase new args are added and plugins don't provide them */
+		if (!a && te[i].num_args > 0)
+			a = "";
+		if (!b && te[i].num_args > 1)
+			b = "";
+		if (!c && te[i].num_args > 2)
+			c = "";
+		if (!d && te[i].num_args > 3)
+			d = "";
+
+		text_emit (i, sess, a, b, c, d);
+		return 1;
+	}
+
+	return 0;
+}
+
 void
 pevent_save (char *fn)
 {
