@@ -27,19 +27,19 @@ N_("-%C10-%C11-%O$t$1 sets ban on $2")},
 N_("-%C10-%C11-%O$tChannel $1 created on $2")},
 
 {"Channel DeHalfOp", pevt_chandehop_help, 2, 0,
-N_("-%C10-%C11-%O$t$1 removes channel half-operator status from $2")},
+N_("-%C10-%C11-%O$t%C11$1%O removes channel half-operator status from %C11$2")},
 
 {"Channel DeOp", pevt_chandeop_help, 2, 0,
-N_("-%C10-%C11-%O$t$1 removes channel operator status from $2")},
+N_("-%C10-%C11-%O$t%C11$1%O removes channel operator status from %C11$2")},
 
 {"Channel DeVoice", pevt_chandevoice_help, 2, 0,
-N_("-%C10-%C11-%O$t$1 removes voice from $2")},
+N_("-%C10-%C11-%O$t%C11$1%O removes voice from %C11$2")},
 
 {"Channel Exempt", pevt_chanexempt_help, 2, 0,
 N_("-%C10-%C11-%O$t$1 sets exempt on $2")},
 
 {"Channel Half-Operator", pevt_chanhop_help, 2, 0,
-N_("-%C10-%C11-%O$t$1 gives channel half-operator status to $2")},
+N_("-%C10-%C11-%O$t%C11$1%O gives channel half-operator status to %C11$2")},
 
 {"Channel INVITE", pevt_chaninvite_help, 2, 0,
 N_("-%C10-%C11-%O$t$1 sets invite on $2")},
@@ -63,7 +63,7 @@ N_("%C2<%C8%B$1%B%C2>%O$t$2%O")},
 N_("%C12-%C13$1/$2%C12-%O$t$3%O")},
 
 {"Channel Operator", pevt_chanop_help, 2, 0,
-N_("-%C10-%C11-%O$t$1 gives channel operator status to $2")},
+N_("-%C10-%C11-%O$t%C11$1%O gives channel operator status to %C11$2")},
 
 {"Channel Remove Exempt", pevt_chanrmexempt_help, 2, 0,
 N_("-%C10-%C11-%O$t$1 removes exempt on $2")},
@@ -87,7 +87,7 @@ N_("-%C10-%C11-%O$t$1 sets channel limit to $2")},
 N_("-%C10-%C11-%O$t$1 removes ban on $2")},
 
 {"Channel Voice", pevt_chanvoice_help, 2, 0,
-N_("-%C10-%C11-%O$t$1 gives voice to $2")},
+N_("-%C10-%C11-%O$t%C11$1%O gives voice to %C11$2")},
 
 {"Connected", pevt_generic_none_help, 0, 0,
 N_("-%C10-%C11-%O$tConnected. Now logging in..")},
@@ -116,8 +116,8 @@ N_("-%C10-%C11-%O$tDCC CHAT to %C11$1%O aborted.")},
 {"DCC CHAT Connect", pevt_dccchatcon_help, 1, 0,
 N_("-%C10-%C11-%O$tDCC CHAT connection established to %C11$1 %C14[%O$2%C14]%O")},
 
-{"DCC CHAT Failed", pevt_dccchaterr_help, 3, 0,
-N_("-%C10-%C11-%O$tDCC CHAT failed. Connection to $1 %C14[%O$2:$3%C14]%O lost.")},
+{"DCC CHAT Failed", pevt_dccchaterr_help, 4, 0,
+N_("-%C10-%C11-%O$tDCC CHAT to %C11$1%O lost. $4.")},
 
 {"DCC CHAT Offer", pevt_generic_nick_help, 1, 0,
 N_("-%C10-%C11-%O$tReceived a DCC CHAT offer from $1")},
@@ -155,8 +155,8 @@ N_("-%C10-%C11-%O$tDCC RECV %C11$1%O from %C11$3%O complete %C14[%C11$4%O cps%C1
 {"DCC RECV Connect", pevt_dcccon_help, 3, 0,
 N_("-%C10-%C11-%O$tDCC RECV connection established to %C11$1 %C14[%O$2%C14]%O")},
 
-{"DCC RECV Failed", pevt_dccrecverr_help, 3, 0,
-N_("-%C10-%C11-%O$tDCC RECV $1 ($2) failed. Connection to $3 lost.")},
+{"DCC RECV Failed", pevt_dccrecverr_help, 4, 0,
+N_("-%C10-%C11-%O$tDCC RECV %C11$1%O from %C11$3%O failed. $4.")},
 
 {"DCC RECV File Open Error", pevt_generic_file_help, 1, 0,
 N_("-%C10-%C11-%O$tDCC RECV: Cannot open $1 for writing - aborting.")},
@@ -176,10 +176,10 @@ N_("-%C10-%C11-%O$tDCC SEND %C11$1%O to %C11$2%O complete %C14[%C11$3%O cps%C14]
 {"DCC SEND Connect", pevt_dcccon_help, 3, 0,
 N_("-%C10-%C11-%O$tDCC SEND connection established to %C11$1 %C14[%O$2%C14]%O")},
 
-{"DCC SEND Failed", pevt_dccsendfail_help, 2, 0,
-N_("-%C10-%C11-%O$tDCC SEND %C11$1%O failed. Connection to %C11$2%O lost.")},
+{"DCC SEND Failed", pevt_dccsendfail_help, 3, 0,
+N_("-%C10-%C11-%O$tDCC SEND %C11$1%O to %C11$2%O failed. $3")},
 
-{"DCC SEND Offer", pevt_dccsendoffer_help, 3, 0,
+{"DCC SEND Offer", pevt_dccsendoffer_help, 4, 0,
 N_("-%C10-%C11-%O$t%C11$1 %Chas offered %C11$2 %C(%C11$3 %Cbytes)")},
 
 {"DCC Stall", pevt_dccstall_help, 3, 0,
