@@ -22,7 +22,6 @@
 #include <stdlib.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <unistd.h>
 #ifdef WIN32
 #include <sys/timeb.h>
 #include <winsock.h>
@@ -278,25 +277,13 @@ nocasestrstr (char *s, char *wanted)
 char *
 errorstring (int err)
 {
-	static char tbuf[16];
 	switch (err)
 	{
 	case -1:
 		return "";
 	case 0:
 		return _("Remote host closed socket");
-#ifndef WIN32
-	case ECONNREFUSED:
-		return _("Connection refused");
-	case ENETUNREACH:
-	case EHOSTUNREACH:
-		return _("No route to host");
-	case ETIMEDOUT:
-		return _("Connection timed out");
-	case EADDRNOTAVAIL:
-		return _("Cannot assign that address");
-	case ECONNRESET:
-#else
+#ifdef WIN32
 	case WSAECONNREFUSED:
 		return _("Connection refused");
 	case WSAENETUNREACH:
@@ -307,11 +294,18 @@ errorstring (int err)
 	case WSAEADDRNOTAVAIL:
 		return _("Cannot assign that address");
 	case WSAECONNRESET:
-#endif
 		return _("Connection reset by peer");
+	default:
+		{
+			static char tbuf[16];
+			sprintf (tbuf, "(%d)", err);
+			return tbuf;
+		}
+#else
+	default:
+		return strerror (err);
+#endif
 	}
-	sprintf (tbuf, "%d", err);
-	return tbuf;
 }
 
 int
@@ -500,10 +494,10 @@ get_cpu_str (void)
 
 	mhz = get_mhz ();
 	if (mhz)
-		sprintf (verbuf, "Windoze %ld.%ld [i%1d86/%dMHz]",
+		sprintf (verbuf, "Windows %ld.%ld [i%d86/%dMHz]",
 			osvi.dwMajorVersion, osvi.dwMinorVersion, si.wProcessorLevel, mhz);
 	else
-		sprintf (verbuf, "Windoze %ld.%ld [i%1d86]",
+		sprintf (verbuf, "Windows %ld.%ld [i%d86]",
 			osvi.dwMajorVersion, osvi.dwMinorVersion, si.wProcessorLevel);
 
 	return verbuf;
@@ -671,7 +665,7 @@ for_files (char *dirname, char *mask, void callback (char *file))
 	}
 }
 
-void
+/*void
 tolowerStr (char *str)
 {
 	while (*str)
@@ -679,7 +673,7 @@ tolowerStr (char *str)
 		*str = rfc_tolower (*str);
 		str++;
 	}
-}
+}*/
 
 /* thanks BitchX */
 
