@@ -250,6 +250,7 @@ new_session (server *serv, char *from, int type)
 	sess->server = serv;
 	sess->logfd = -1;
 	sess->type = type;
+	sess->hide_join_part = prefs.confmode;
 
 	if (from != NULL)
 		safe_strcpy (sess->channel, from, CHANLEN);
@@ -272,11 +273,11 @@ set_server_defaults (server *serv)
 		free (serv->nick_prefixes);
 	if (serv->nick_modes)
 		free (serv->nick_modes);
-	if (serv->encoding)
+	/*if (serv->encoding)
 	{
 		free (serv->encoding);
 		serv->encoding = NULL;
-	}
+	}*/
 
 	serv->chantypes = strdup ("#&!+");
 	serv->chanmodes = strdup ("beI,k,l");
@@ -676,9 +677,9 @@ find_away_message (struct server *serv, char *nick)
 	"NAME Clear\n"				"CMD clear\n\n"
 
 #define defaultconf_replace \
-	"NAME teh\n"				"CMD the\n\n"\
-	"NAME r\n"					"CMD are\n\n"\
-	"NAME u\n"					"CMD you\n\n"
+	"NAME teh\n"				"CMD the\n\n"
+/*	"NAME r\n"					"CMD are\n\n"\
+	"NAME u\n"					"CMD you\n\n"*/
 
 #define defaultconf_commands \
 	"NAME ACTION\n"		"CMD me &2\n\n"\
@@ -895,6 +896,10 @@ xchat_exit (void)
 	ignore_save ();
 	free_sessions ();
 	fe_exit ();
+
+#ifdef WIN32
+	WSACleanup ();
+#endif
 }
 
 #ifndef WIN32
