@@ -42,38 +42,52 @@
 #include "palette.h"
 #include "maingui.h"
 
+#ifdef USE_GNOME
+#include <libgnomevfs/gnome-vfs-mime-utils.h>
+#endif
+
 struct dccwindow
 {
 	GtkWidget *window;
 	GtkWidget *list;
 };
 
+struct my_dcc_send
+{
+	struct session *sess;
+	char *nick;
+	int maxcps;
+};
+
 static struct dccwindow dccrwin;	/* recv */
 static struct dccwindow dccswin;	/* send */
 static struct dccwindow dcccwin;	/* chat */
 
 
 static void
-dcc_send_filereq_done (struct session *sess, char *nick, char *file)
+dcc_send_filereq_done (struct my_dcc_send *mdc, void *dummy, char *file)
 {
 	char tbuf[400];
 
 	if (file)
-	{
-		dcc_send (sess, tbuf, nick, file);
-		free (file);
-	}
-	free (nick);
+		dcc_send (mdc->sess, tbuf, mdc->nick, file, mdc->maxcps);
+	free (mdc->nick);
+	free (mdc);
 }
 
 void
-fe_dcc_send_filereq (struct session *sess, char *nick)
+fe_dcc_send_filereq (struct session *sess, char *nick, int maxcps)
 {
 	char tbuf[128];
+	struct my_dcc_send *mdc;
+	
+	mdc = malloc (sizeof (*mdc));
+	mdc->sess = sess;
+	mdc->nick = strdup (nick);
+	mdc->maxcps = maxcps;
 
-	nick = strdup (nick);
 	snprintf (tbuf, sizeof tbuf, _("Send file to %s"), nick);
-	gtkutil_file_req (tbuf, dcc_send_filereq_done, sess, nick, FALSE);
+	gtkutil_file_req (tbuf, dcc_send_filereq_done, mdc, NULL, FALSE);
 }
 
 void
@@ -118,7 +132,7 @@ fe_dcc_update_recv (struct DCC *dcc)
 #ifdef USE_GNOME
 	if (dcc->dccstat == STAT_DONE)
 		gtk_clist_set_text (GTK_CLIST (dccrwin.list), row, 8,
-								  (char *) gnome_mime_type_of_file (dcc->destfile));
+								  (char *) gnome_vfs_get_mime_type (dcc->destfile));
 #endif
 	gtk_clist_thaw (GTK_CLIST (dccrwin.list));
 }
@@ -209,7 +223,7 @@ fe_dcc_update_recv_win (void)
 			nnew[7] = dcc->nick;
 #ifdef USE_GNOME
 			if (dcc->dccstat == STAT_DONE)
-				nnew[8] = (char *) gnome_mime_type_of_file (dcc->destfile);
+				nnew[8] = (char *) gnome_vfs_get_mime_type (dcc->destfile);
 			else
 				nnew[8] = "";
 #endif
@@ -252,12 +266,14 @@ dcc_info (struct DCC *dcc)
 				 "      Size: %u\n"
 				 "      Port: %d\n"
 				 " IP Number: %s\n"
-				 "Start Time: %s"),
+				 "Start Time: %s"
+				 "   Max CPS: %d\n"),
 				 dcc->file,
 				 dcc->nick,
 				 dcc->size,
 				 dcc->port,
-				 net_ip (dcc->addr), ctime (&dcc->starttime));
+				 net_ip (dcc->addr), ctime (&dcc->starttime),
+				 dcc->maxcps);
 	gtkutil_simpledialog (tbuf);
 }
 
@@ -327,37 +343,16 @@ open_clicked (void)
 {
 	int row;
 	struct DCC *dcc;
-	char *mime_type;
-	char *mime_prog;
-	char *tmp;
 
 	row = gtkutil_clist_selection (dccrwin.list);
-	if (row != -1)
-	{
-		dcc = gtk_clist_get_row_data (GTK_CLIST (dccrwin.list), row);
-		if (dcc && dcc->dccstat == STAT_DONE)
-		{
-			mime_type = (char *) gnome_mime_type (dcc->destfile);
-			if (mime_type)
-			{
-				mime_prog = (char *) gnome_mime_program (mime_type);
-				if (mime_prog)
-				{
-					mime_prog = strdup (mime_prog);
-					tmp = strstr (mime_prog, "%f");
-					if (tmp)
-					{
-						tmp[1] = 's';
-						tmp = malloc (strlen (dcc->destfile) + strlen (mime_prog));
-						sprintf (tmp, mime_prog, dcc->destfile);
-						xchat_exec (tmp);
-						free (tmp);
-					}
-					free (mime_prog);
-				}
-			}
-		}
-	}
+	if (row == -1)
+		return;
+
+	dcc = gtk_clist_get_row_data (GTK_CLIST (dccrwin.list), row);
+	if (dcc || dcc->dccstat == STAT_DONE)
+		return;
+
+	/* do something with dcc->destfile */
 }
 
 #endif
@@ -392,7 +387,7 @@ fe_dcc_open_recv_win (int passive)
 	}
 	dccrwin.window =
 			  mg_create_generic_tab ("dccrecv", _("X-Chat: File Receive List"),
-							FALSE, TRUE, close_dcc_recv_window, NULL, 601, 0, &vbox);
+						FALSE, TRUE, close_dcc_recv_window, NULL, 601, 0, &vbox, 0);
 
 #ifdef USE_GNOME
 	dccrwin.list = gtkutil_clist_new (9, titles, vbox, GTK_POLICY_ALWAYS,
@@ -419,15 +414,15 @@ fe_dcc_open_recv_win (int passive)
 	gtk_box_pack_start (GTK_BOX (vbox), bbox, FALSE, FALSE, 2);
 	gtk_widget_show (bbox);
 
-	gtkutil_button (bbox, GTK_STOCK_CANCEL, 0, abort_clicked, 0, _("Abort"), 0, 0);
+	gtkutil_button (bbox, GTK_STOCK_CANCEL, 0, abort_clicked, 0, _("Abort"));
 
-	gtkutil_button (bbox, GTK_STOCK_APPLY, 0, accept_clicked, 0, _("Accept"), 0, 0);
+	gtkutil_button (bbox, GTK_STOCK_APPLY, 0, accept_clicked, 0, _("Accept"));
 
-	gtkutil_button (bbox, GTK_STOCK_REFRESH, 0, resume_clicked, 0, _("Resume"), 0, 0);
+	gtkutil_button (bbox, GTK_STOCK_REFRESH, 0, resume_clicked, 0, _("Resume"));
 
-	gtkutil_button (bbox, GTK_STOCK_DIALOG_INFO, 0, info_clicked, 0, _("Info"), 0, 0);
+	gtkutil_button (bbox, GTK_STOCK_DIALOG_INFO, 0, info_clicked, 0, _("Info"));
 #ifdef USE_GNOME
-	gtkutil_button (bbox, GTK_STOCK_OPEN, 0, open_clicked, 0, _("Open"), 0, 0);
+	gtkutil_button (bbox, GTK_STOCK_OPEN, 0, open_clicked, 0, _("Open"));
 #endif
 	gtk_widget_show (dccrwin.window);
 	fe_dcc_update_recv_win ();
@@ -572,7 +567,7 @@ fe_dcc_open_send_win (int passive)
 
 	dccswin.window =
 			  mg_create_generic_tab ("dccsend", _("X-Chat: File Send List"),
-							FALSE, TRUE, close_dcc_send_window, NULL, 595, 0, &vbox);
+						FALSE, TRUE, close_dcc_send_window, NULL, 595, 0, &vbox, 0);
 
 	dccswin.list = gtkutil_clist_new (9, titles, vbox, GTK_POLICY_ALWAYS,
 												 send_row_selected, 0,
@@ -593,8 +588,8 @@ fe_dcc_open_send_win (int passive)
 	gtk_box_pack_start (GTK_BOX (vbox), bbox, FALSE, FALSE, 2);
 	gtk_widget_show (bbox);
 
-	gtkutil_button (bbox, GTK_STOCK_CANCEL, 0, abort_send_clicked, 0, _("Abort"), 0, 0);
-	gtkutil_button (bbox, GTK_STOCK_DIALOG_INFO, 0, info_send_clicked, 0, _("Info"), 0, 0);
+	gtkutil_button (bbox, GTK_STOCK_CANCEL, 0, abort_send_clicked, 0, _("Abort"));
+	gtkutil_button (bbox, GTK_STOCK_DIALOG_INFO, 0, info_send_clicked, 0, _("Info"));
 
 	gtk_widget_show (dccswin.window);
 	fe_dcc_update_send_win ();
@@ -703,7 +698,7 @@ fe_dcc_open_chat_win (int passive)
 
 	dcccwin.window =
 			  mg_create_generic_tab ("dccchat", _("X-Chat: DCC Chat List"),
-							FALSE, TRUE, close_dcc_chat_window, NULL, 550, 0, &vbox);
+						FALSE, TRUE, close_dcc_chat_window, NULL, 550, 0, &vbox, 0);
 
 	dcccwin.list = gtkutil_clist_new (5, titles, vbox, GTK_POLICY_ALWAYS,
 												 chat_row_selected, 0,
@@ -718,8 +713,8 @@ fe_dcc_open_chat_win (int passive)
 	gtk_box_pack_start (GTK_BOX (vbox), bbox, FALSE, FALSE, 2);
 	gtk_widget_show (bbox);
 
-	gtkutil_button (bbox, GTK_STOCK_CANCEL, 0, abort_chat_clicked, 0, _("Abort"), 0, 0);
-	gtkutil_button (bbox, GTK_STOCK_APPLY, 0, accept_chat_clicked, 0, _("Accept"), 0, 0);
+	gtkutil_button (bbox, GTK_STOCK_CANCEL, 0, abort_chat_clicked, 0, _("Abort"));
+	gtkutil_button (bbox, GTK_STOCK_APPLY, 0, accept_chat_clicked, 0, _("Accept"));
 
 	gtk_widget_show (dcccwin.window);
 	fe_dcc_update_chat_win ();
