@@ -163,7 +163,7 @@ static const struct key_action key_actions[KEY_MAX_ACTIONS + 1] = {
 	{key_action_comp_chng, "Change Selected Nick",
 	 N_("This command scrolls up and down through the list of nicks. If Data 1 is set to anything it will scroll up, else it scrolls down")},
 	{key_action_replace, "Check For Replace",
-	 N_("This command checks the last work entered in the entry against the replace list and replaces it if it finds a match")},
+	 N_("This command checks the last word entered in the entry against the replace list and replaces it if it finds a match")},
 	{key_action_move_tab_left, "Move front tab left",
 	 N_("This command move the front tab left by one")},
 	{key_action_move_tab_right, "Move front tab right",
@@ -342,7 +342,8 @@ key_load_defaults ()
 {
 		/* This is the default config */
 #define defcfg \
-		"A\nminus\nChange Page\nD1:-1\nD2:Relative\n\n"\
+		"C\nPrior\nChange Page\nD1:-1\nD2:Relative\n\n"\
+		"C\nNext\nChange Page\nD1:1\nD2:Relative\n\n"\
 		"A\n9\nChange Page\nD1:9\nD2!\n\n"\
 		"A\n8\nChange Page\nD1:8\nD2!\n\n"\
 		"A\n7\nChange Page\nD1:7\nD2!\n\n"\
@@ -364,7 +365,7 @@ key_load_defaults ()
 		"None\nTab\nComplete nick/command\nD1!\nD2!\n\n"\
 		"None\nspace\nCheck For Replace\nD1!\nD2!\n\n"\
 		"None\nReturn\nCheck For Replace\nD1!\nD2!\n\n"\
-		"A\nequal\nChange Page\nD1:1\nD2:Relative\n\n"\
+		"None\nKP_Enter\nCheck For Replace\nD1!\nD2!\n\n"\
 		"C\nTab\nComplete nick/command\nD1:Up\nD2!\n\n"\
 		"A\nLeft\nMove front tab left\nD1!\nD2!\n\n"\
 		"A\nRight\nMove front tab right\nD1!\nD2!\n\n"
