@@ -129,11 +129,39 @@ mg_set_access_icon (session_gui *gui, GdkPixbuf *pix)
 	}
 }
 
+static gboolean
+mg_inputbox_focus (GtkWidget *widget, GdkEventFocus *event, session_gui *gui)
+{
+	GSList *list;
+	session *sess;
+
+	if (gui->is_tab)
+		return FALSE;
+
+	list = sess_list;
+	while (list)
+	{
+		sess = list->data;
+		if (sess->gui == gui)
+		{
+			current_sess = sess;
+			if (!sess->server->server_session)
+				sess->server->server_session = sess;
+			break;
+		}
+		list = list->next;
+	}
+
+	return FALSE;
+}
+
 static void
-mg_inputbox_cb (GtkWidget *igad, gpointer userdata)
+mg_inputbox_cb (GtkWidget *igad, session_gui *gui)
 {
 	char *cmd = GTK_ENTRY (igad)->text;
 	static int ignore = FALSE;
+	GSList *list;
+	session *sess = NULL;
 
 	if (ignore)
 		return;
@@ -148,7 +176,26 @@ mg_inputbox_cb (GtkWidget *igad, gpointer userdata)
 	gtk_entry_set_text (GTK_ENTRY (igad), "");
 	ignore = FALSE;
 
-	handle_multiline (current_sess, cmd, TRUE, FALSE);
+	/* where did this event come from? */
+	if (gui->is_tab)
+	{
+		sess = current_tab;
+	} else
+	{
+		list = sess_list;
+		while (list)
+		{
+			sess = list->data;
+			if (sess->gui == gui)
+				break;
+			list = list->next;
+		}
+		if (!list)
+			sess = NULL;
+	}
+
+	if (sess)
+		handle_multiline (sess, cmd, TRUE, FALSE);
 	free (cmd);
 }
 
@@ -395,8 +442,9 @@ mg_set_topic_tip (session *sess)
 {
 	char buf[512];
 
-	if (sess->type == SESS_CHANNEL)
+	switch (sess->type)
 	{
+	case SESS_CHANNEL:
 		if (sess->topic)
 		{
 			snprintf (buf, sizeof (buf), _("Topic for %s is: %s"), sess->channel,
@@ -404,6 +452,9 @@ mg_set_topic_tip (session *sess)
 			add_tip (sess->gui->topic_entry, buf);
 		} else
 			add_tip (sess->gui->topic_entry, _("No topic is set"));
+		break;
+	default:
+		add_tip (sess->gui->topic_entry, GTK_ENTRY (sess->gui->topic_entry)->text);
 	}
 }
 
@@ -414,15 +465,19 @@ mg_populate_userlist (session *sess)
 {
 	session_gui *gui;
 
-	sess = current_sess;
-	gui = sess->gui;
+	if (!sess)
+		sess = current_tab;
 
-	if (sess->type == SESS_DIALOG)
-		mg_set_access_icon (sess->gui, NULL);
-	else
-		mg_set_access_icon (sess->gui, get_user_icon (sess->server, sess->res->myself));
-	userlist_show (sess);
-	userlist_set_value (sess->gui->user_tree, sess->res->old_ul_value);
+	if (is_session (sess))
+	{
+		gui = sess->gui;
+		if (sess->type == SESS_DIALOG)
+			mg_set_access_icon (sess->gui, NULL);
+		else
+			mg_set_access_icon (sess->gui, get_user_icon (sess->server, sess->res->myself));
+		userlist_show (sess);
+		userlist_set_value (sess->gui->user_tree, sess->res->old_ul_value);
+	}
 
 	ul_tag = 0;
 	return 0;
@@ -497,8 +552,14 @@ mg_populate (session *sess)
 		gtk_button_set_label (GTK_BUTTON (gui->nick_label), sess->server->nick);
 
 	/* this is slow, so make it a timeout event */
-	if (ul_tag == 0)
-		ul_tag = g_idle_add ((GSourceFunc)mg_populate_userlist, 0);
+	if (!gui->is_tab)
+	{
+		mg_populate_userlist (sess);
+	} else
+	{
+		if (ul_tag == 0)
+			ul_tag = g_idle_add ((GSourceFunc)mg_populate_userlist, NULL);
+	}
 
 	fe_userlist_numbers (sess);
 
@@ -752,7 +813,7 @@ mg_add_chan (session *sess)
 		name = sess->channel;
 
 	sess->res->tab = tab_group_add (sess->gui->tabs_box, name, sess->server,
-											  sess, mg_tab_press_cb);
+											  sess, mg_tab_press_cb, prefs.truncchans);
 	g_object_set_data (G_OBJECT (sess->res->tab), "sess", sess);
 
 	if (newmsg_list == NULL)
@@ -1153,6 +1214,14 @@ mg_limit_entry_cb (GtkWidget * igad, gpointer userdata)
 	}
 }
 
+static void
+mg_apply_entry_style (GtkWidget *entry)
+{
+	gtk_widget_modify_base (entry, GTK_STATE_NORMAL, &colors[19]);
+	gtk_widget_modify_text (entry, GTK_STATE_NORMAL, &colors[18]);
+	gtk_widget_modify_font (entry, input_style->font_desc);
+}
+
 static void
 mg_create_chanmodebuttons (session_gui *gui, GtkWidget *box)
 {
@@ -1172,19 +1241,27 @@ mg_create_chanmodebuttons (session_gui *gui, GtkWidget *box)
 
 	gui->flag_k = mg_create_flagbutton (_("Keyword"), box, "K");
 	gui->key_entry = gtk_entry_new ();
+	gtk_widget_set_name (gui->key_entry, "xchat-inputbox");
 	gtk_entry_set_max_length (GTK_ENTRY (gui->key_entry), 16);
 	gtk_widget_set_size_request (gui->key_entry, 30, -1);
 	gtk_box_pack_start (GTK_BOX (box), gui->key_entry, 0, 0, 0);
 	g_signal_connect (G_OBJECT (gui->key_entry), "activate",
 							G_CALLBACK (mg_key_entry_cb), NULL);
 
+	if (prefs.style_inputbox)
+		mg_apply_entry_style (gui->key_entry);
+
 	gui->flag_l = mg_create_flagbutton (_("User Limit"), box, "L");
 	gui->limit_entry = gtk_entry_new ();
+	gtk_widget_set_name (gui->limit_entry, "xchat-inputbox");
 	gtk_entry_set_max_length (GTK_ENTRY (gui->limit_entry), 10);
 	gtk_widget_set_size_request (gui->limit_entry, 30, -1);
 	gtk_box_pack_start (GTK_BOX (box), gui->limit_entry, 0, 0, 0);
 	g_signal_connect (G_OBJECT (gui->limit_entry), "activate",
 							G_CALLBACK (mg_limit_entry_cb), NULL);
+
+	if (prefs.style_inputbox)
+		mg_apply_entry_style (gui->limit_entry);
 }
 
 static void
@@ -1210,6 +1287,11 @@ mg_dialog_button_cb (GtkWidget *wid, char *cmd)
 	auto_insert (buf, cmd, 0, 0, "", "", "", "", "", current_sess->channel);
 
 	handle_command (current_sess, buf, TRUE);
+
+	/* dirty trick to avoid auto-selection */
+	GTK_ENTRY (current_sess->gui->input_box)->editable = 0;
+	gtk_widget_grab_focus (current_sess->gui->input_box);
+	GTK_ENTRY (current_sess->gui->input_box)->editable = 1;
 }
 
 static void
@@ -1258,10 +1340,14 @@ mg_create_topicbar (session *sess, GtkWidget *box, char *name)
 	}
 
 	gui->topic_entry = topic = gtk_entry_new ();
+	gtk_widget_set_name (topic, "xchat-inputbox");
 	gtk_container_add (GTK_CONTAINER (hbox), topic);
 	g_signal_connect (G_OBJECT (topic), "activate",
 							G_CALLBACK (mg_topic_cb), 0);
 
+	if (prefs.style_inputbox)
+		mg_apply_entry_style (topic);
+
 	gui->topicbutton_box = bbox = gtk_hbox_new (FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (hbox), bbox, 0, 0, 0);
 	mg_create_chanmodebuttons (gui, bbox);
@@ -1292,6 +1378,12 @@ mg_word_clicked (GtkWidget *xtext, char *word, GdkEventButton *even,
 
 	if (even->button == 1)			/* left button */
 	{
+		if (word == NULL)
+		{
+			mg_focus (sess);
+			return;
+		}
+
 		if (even->state & GDK_CONTROL_MASK)
 		{
 			switch (mg_word_check (xtext, word))
@@ -1452,12 +1544,15 @@ mg_create_meters (session_gui *gui, GtkWidget *box)
 }
 
 static void
-mg_create_userlist (session_gui *gui, GtkWidget *box)
+mg_create_userlist (session_gui *gui, GtkWidget *box, int pack)
 {
 	GtkWidget *frame, *ulist, *vbox;
 
 	vbox = gtk_vbox_new (0, 0);
-	gtk_box_pack_start (GTK_BOX (box), vbox, 0, 0, 0);
+	if (pack)
+		gtk_box_pack_start (GTK_BOX (box), vbox, 0, 0, 0);
+	else
+		gtk_container_add (GTK_CONTAINER (box), vbox);
 
 	frame = gtk_frame_new (NULL);
 	gtk_box_pack_start (GTK_BOX (vbox), frame, 0, 0, 1);
@@ -1484,37 +1579,37 @@ mg_create_userlist (session_gui *gui, GtkWidget *box)
 static void
 mg_create_center (session *sess, session_gui *gui, GtkWidget *box)
 {
-	GtkWidget *vbox, *hbox, *tbox, *paned;
+	GtkWidget *vbox, *hbox, *paned;
+
+	hbox = gtk_hbox_new (FALSE, 1);
 
 	if (prefs.paned_userlist)
 	{
 		paned = gtk_hpaned_new ();
-
-		hbox = gtk_hbox_new (FALSE, 1);
 		gtk_paned_add1 (GTK_PANED (paned), hbox);
 
 		vbox = gtk_vbox_new (FALSE, 1);
 		gtk_container_add (GTK_CONTAINER (hbox), vbox);
 
-		tbox = gtk_hbox_new (FALSE, 0);
-		gtk_paned_add2 (GTK_PANED (paned), tbox);
-
 		gtk_container_add (GTK_CONTAINER (box), paned);
 
 		mg_create_textarea (gui, vbox);
-		mg_create_userlist (gui, tbox);
 		mg_create_entry (sess, vbox);
 
+		hbox = gtk_hbox_new (FALSE, 1);
+		gtk_paned_add2 (GTK_PANED (paned), hbox);
+
+		mg_create_userlist (gui, hbox, FALSE);
+
 	} else
 	{
-		hbox = gtk_hbox_new (FALSE, 1);
 		gtk_container_add (GTK_CONTAINER (box), hbox);
 
 		vbox = gtk_vbox_new (FALSE, 1);
 		gtk_container_add (GTK_CONTAINER (hbox), vbox);
 
 		mg_create_textarea (gui, vbox);
-		mg_create_userlist (gui, hbox);
+		mg_create_userlist (gui, hbox, TRUE);
 		mg_create_entry (sess, vbox);
 	}
 }
@@ -1628,6 +1723,9 @@ mg_nickclick_cb (GtkWidget *button, gpointer userdata)
 static void
 mg_set_tabs_pos (session_gui *gui, int pos)
 {
+	GtkOrientation orientation;
+	GtkWidget *new_tabs_box;
+
 	if (!gui)
 	{
 		gui = mg_gui;
@@ -1635,26 +1733,50 @@ mg_set_tabs_pos (session_gui *gui, int pos)
 			return;
 	}
 
+	gtk_widget_ref (gui->tabs_box);
+
+	if (pos != 4)
+		gtk_container_remove (GTK_CONTAINER (gui->main_table), gui->tabs_box);
+
+	orientation = tab_group_get_orientation (gui->tabs_box);
+	if ((pos == 0 || pos == 1) && orientation == GTK_ORIENTATION_VERTICAL)
+	{
+		new_tabs_box = tab_group_set_orientation (gui->tabs_box, FALSE);
+		gtk_widget_unref (gui->tabs_box);
+		gui->tabs_box = new_tabs_box;
+	} else if((pos == 2 || pos == 3) && orientation == GTK_ORIENTATION_HORIZONTAL) {
+		new_tabs_box = tab_group_set_orientation (gui->tabs_box, TRUE);
+		gtk_widget_unref (gui->tabs_box);
+		gui->tabs_box = new_tabs_box;
+	}
+
 	gtk_widget_show (gui->tabs_box);
 
 	switch (pos)
 	{
 	case 0: /* bottom */
-		gtk_box_reorder_child (GTK_BOX (gui->main_vbox), gui->tabs_box, 9);
+		gtk_table_attach (GTK_TABLE (gui->main_table), gui->tabs_box,
+							1, 2, 2, 3, GTK_FILL, GTK_FILL, 0, 0);
 		break;
 	case 1: /* top */
-		gtk_box_reorder_child (GTK_BOX (gui->main_vbox), gui->tabs_box, 1);
+		gtk_table_attach (GTK_TABLE (gui->main_table), gui->tabs_box,
+							1, 2, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
 		break;
-#if 0
 	case 2: /* left */
+		gtk_table_attach (GTK_TABLE (gui->main_table), gui->tabs_box,
+							0, 1, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
 		break;
 	case 3: /* right */
+		gtk_table_attach (GTK_TABLE (gui->main_table), gui->tabs_box,
+							2, 3, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
 		break;
-#endif
-	case 2: /* hidden */
+	case 4: /* hidden */
 		gtk_widget_hide (gui->tabs_box);
 		break;
 	}
+
+	if (orientation == tab_group_get_orientation (gui->tabs_box))
+		gtk_widget_unref (gui->tabs_box);
 }
 
 static void
@@ -1684,13 +1806,15 @@ mg_create_entry (session *sess, GtkWidget *box)
 	gtk_entry_set_max_length (GTK_ENTRY (gui->input_box), 2048);
 	g_signal_connect (G_OBJECT (entry), "key_press_event",
 							G_CALLBACK (key_handle_key_press), NULL);
+	g_signal_connect (G_OBJECT (entry), "focus_in_event",
+							G_CALLBACK (mg_inputbox_focus), gui);
 	g_signal_connect (G_OBJECT (entry), "activate",
-							G_CALLBACK (mg_inputbox_cb), NULL);
+							G_CALLBACK (mg_inputbox_cb), gui);
 	gtk_container_add (GTK_CONTAINER (hbox), entry);
 	gtk_widget_grab_focus (entry);
 
 	if (prefs.style_inputbox)
-		gtk_widget_set_style (entry, input_style);
+		mg_apply_entry_style (entry);
 
 	but = gtkutil_button (hbox, GTK_STOCK_GO_UP, _("Channel Options"),
 								 mg_upbutton_cb, 0, 0);
@@ -1727,7 +1851,8 @@ mg_create_tabs (session_gui *gui, GtkWidget *box)
 		vert = TRUE;
 
 	gui->tabs_box = tab_group_new (mg_switch_tab_cb, vert);
-	gtk_box_pack_start (GTK_BOX (box), gui->tabs_box, 0, 0, 0);
+	gtk_table_attach (GTK_TABLE (gui->main_table), gui->tabs_box,
+						1, 2, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
 }
 
 static gboolean
@@ -1896,6 +2021,7 @@ mg_create_tabwindow (session *sess)
 {
 	GtkWidget *win;
 	GtkWidget *vbox;
+	GtkWidget *table;
 	GtkWidget *book;
 
 	win = gtkutil_window_new ("X-Chat ["VERSION"]", prefs.mainwindow_width,
@@ -1921,10 +2047,14 @@ mg_create_tabwindow (session *sess)
 	sess->gui->main_vbox = vbox = gtk_vbox_new (FALSE, 1);
 	gtk_container_add (GTK_CONTAINER (win), vbox);
 
+	sess->gui->main_table = table = gtk_table_new (3, 3, FALSE);
+	gtk_container_add (GTK_CONTAINER (vbox), table);
+
 	sess->gui->note_book = book = gtk_notebook_new ();
 	gtk_notebook_set_show_tabs (GTK_NOTEBOOK (book), FALSE);
 	gtk_notebook_set_show_border (GTK_NOTEBOOK (book), FALSE);
-	gtk_container_add (GTK_CONTAINER (vbox), book);
+	gtk_table_attach (GTK_TABLE (table), book, 1, 2, 1, 2,
+						GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
 
 	mg_create_irctab (sess, book);
 	mg_create_tabs (sess->gui, vbox);
@@ -1932,13 +2062,14 @@ mg_create_tabwindow (session *sess)
 
 	mg_focus (sess);
 
-	if (prefs.tabs_position != 0)
+	if (prefs.tabs_position != 1)
 		mg_set_tabs_pos (sess->gui, prefs.tabs_position);
 
 	if (!prefs.hidemenu)
 		gtk_widget_show (sess->gui->menu);
 
 	gtk_widget_show (vbox);
+	gtk_widget_show (table);
 	gtk_widget_show_all (book);
 
 	if (prefs.hideuserlist)
@@ -1978,7 +2109,7 @@ mg_add_generic_tab (char *name, char *title, void *family, GtkWidget *box)
 	gtk_notebook_append_page (GTK_NOTEBOOK (mg_gui->note_book), box, NULL);
 	gtk_widget_show (box);
 
-	but = tab_group_add (mg_gui->tabs_box, name, family, NULL, mg_tab_press_cb);
+	but = tab_group_add (mg_gui->tabs_box, name, family, NULL, mg_tab_press_cb, prefs.truncchans);
 	g_object_set_data (G_OBJECT (but), "title", strdup (title));
 	g_object_set_data (G_OBJECT (but), "box", box);
 	g_object_set_data (G_OBJECT (but), "sess", NULL);
@@ -2028,7 +2159,7 @@ fe_clear_channel (session *sess)
 	}
 	else
 		strcpy (tbuf, _("<none>"));
-	tab_rename (sess->res->tab, tbuf);
+	tab_rename (sess->res->tab, tbuf, prefs.truncchans);
 
 	if (!sess->gui->is_tab || sess == current_tab)
 	{
@@ -2152,21 +2283,7 @@ fe_set_away (server *serv)
 void
 fe_set_channel (session *sess)
 {
-	char *buf;
-
-	if (prefs.truncchans > 2 && g_utf8_strlen (sess->channel, -1) > prefs.truncchans)
-	{
-		/* truncate long channel names */
-		buf = malloc (strlen (sess->channel) + 4);
-		strcpy (buf, sess->channel);
-		g_utf8_offset_to_pointer(buf, prefs.truncchans)[0] = 0;
-		strcat (buf, "..");
-		tab_rename (sess->res->tab, buf);
-		free (buf);
-	} else
-	{
-		tab_rename (sess->res->tab, sess->channel);
-	}
+	tab_rename (sess->res->tab, sess->channel, prefs.truncchans);
 }
 
 void
