@@ -1,19 +1,26 @@
 #define GTK_DISABLE_DEPRECATED
 
+#include <string.h>
+#include <stdlib.h>
+
+#include <gtk/gtkarrow.h>
 #include <gtk/gtkhbox.h>
+#include <gtk/gtkvbox.h>
 #include <gtk/gtktogglebutton.h>
 #include <gtk/gtkwidget.h>
 #include <gtk/gtkcontainer.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtksignal.h>
+#include <gtk/gtkhseparator.h>
+#include <gtk/gtkvbox.h>
 #include <gtk/gtkviewport.h>
 #include <gtk/gtkvseparator.h>
 
 /* keep this code generic, don't include xchat.h! */
 
 #include "tabs.h"
-#undef TABS_SPREAD		/* left justify tabs instead */
 
+#undef ALPHA_SORT
 
 /* ignore "toggled" signal? */
 static int ignore_toggle = FALSE;
@@ -25,6 +32,7 @@ static int ignore_toggle = FALSE;
  *   "c" tab-focus callback function
  *   "foc" currently focused tab
  *   "i" inner hbox (inside the viewport)
+ *   "v" set to 1 if group is vertical type
  *
  * family boxes inside group
  *   "f" family
@@ -44,17 +52,21 @@ static int ignore_toggle = FALSE;
 static void
 tab_viewport_size_request (GtkWidget *widget, GtkRequisition *requisition, gpointer user_data)
 {
-	requisition->width = 1;
-	requisition->height = 1;
+	if (GPOINTER_TO_INT (g_object_get_data (G_OBJECT (user_data), "v")) == 0)
+		requisition->width = 1;
+	else
+		requisition->height = 1;
 }
 
 static gint
-tab_search_offset (GtkWidget *inner, gint start_offset, gboolean forward)
+tab_search_offset (GtkWidget *inner, gint start_offset,
+				   gboolean forward, gboolean vertical)
 {
 	GList *boxes;
 	GList *tabs;
 	GtkWidget *box;
 	GtkWidget *button;
+	gint found;
 
 	boxes = GTK_BOX (inner)->children;
 	if (!forward && boxes)
@@ -77,9 +89,10 @@ tab_search_offset (GtkWidget *inner, gint start_offset, gboolean forward)
 			if (!GTK_IS_TOGGLE_BUTTON (button))
 				continue;
 
-			if ((forward && button->allocation.x > start_offset) ||
-			    (!forward && button->allocation.x < start_offset))
-				return button->allocation.x;
+			found = (vertical ? button->allocation.y : button->allocation.x);
+			if ((forward && found > start_offset) ||
+				(!forward && found < start_offset))
+				return found;
 		}
 	}
 
@@ -87,21 +100,32 @@ tab_search_offset (GtkWidget *inner, gint start_offset, gboolean forward)
 }
 
 static void
-tab_scroll_left_clicked (GtkWidget *widget, GtkWidget *group)
+tab_scroll_left_up_clicked (GtkWidget *widget, GtkWidget *group)
 {
 	GtkAdjustment *adj;
-	gint viewport_width;
+	gint viewport_size;
 	gfloat new_value;
 	GtkWidget *inner;
+	gint vertical;
 	gfloat i;
 
 	inner = g_object_get_data (G_OBJECT (group), "i");
-	adj = gtk_viewport_get_hadjustment (GTK_VIEWPORT (inner->parent));
-	gdk_window_get_geometry (inner->parent->window, 0, 0, &viewport_width, 0, 0);
-	new_value = tab_search_offset (inner, adj->value, 0);
+	vertical = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (group), "v"));
 
-	if (new_value + viewport_width > adj->upper)
-		new_value = adj->upper - viewport_width;
+	if (vertical)
+	{
+		adj = gtk_viewport_get_vadjustment (GTK_VIEWPORT (inner->parent));
+		gdk_window_get_geometry (inner->parent->window, 0, 0, 0, &viewport_size, 0);
+	} else
+	{
+		adj = gtk_viewport_get_hadjustment (GTK_VIEWPORT (inner->parent));
+		gdk_window_get_geometry (inner->parent->window, 0, 0, &viewport_size, 0, 0);
+	}
+
+	new_value = tab_search_offset (inner, adj->value, 0, vertical);
+
+	if (new_value + viewport_size > adj->upper)
+		new_value = adj->upper - viewport_size;
 
 	for (i = adj->value; i > new_value; i -= 0.1)
 	{
@@ -114,21 +138,32 @@ tab_scroll_left_clicked (GtkWidget *widget, GtkWidget *group)
 }
 
 static void
-tab_scroll_right_clicked (GtkWidget *widget, GtkWidget *group)
+tab_scroll_right_down_clicked (GtkWidget *widget, GtkWidget *group)
 {
 	GtkAdjustment *adj;
-	gint viewport_width;
+	gint viewport_size;
 	gfloat new_value;
 	GtkWidget *inner;
+	gint vertical;
 	gfloat i;
 
 	inner = g_object_get_data (G_OBJECT (group), "i");
-	adj = gtk_viewport_get_hadjustment (GTK_VIEWPORT (inner->parent));
-	gdk_window_get_geometry (inner->parent->window, 0, 0, &viewport_width, 0, 0);
-	new_value = tab_search_offset (inner, adj->value, 1);
+	vertical = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (group), "v"));
 
-	if (new_value == 0 || new_value + viewport_width > adj->upper)
-		new_value = adj->upper - viewport_width;
+	if (vertical)
+	{
+		adj = gtk_viewport_get_vadjustment (GTK_VIEWPORT (inner->parent));
+		gdk_window_get_geometry (inner->parent->window, 0, 0, 0, &viewport_size, 0);
+	} else
+	{
+		adj = gtk_viewport_get_hadjustment (GTK_VIEWPORT (inner->parent));
+		gdk_window_get_geometry (inner->parent->window, 0, 0, &viewport_size, 0, 0);
+	}
+
+	new_value = tab_search_offset (inner, adj->value, 1, vertical);
+
+	if (new_value == 0 || new_value + viewport_size > adj->upper)
+		new_value = adj->upper - viewport_size;
 
 	for (i = adj->value; i < new_value; i += 0.1)
 	{
@@ -174,36 +209,51 @@ tab_group_new (void *callback, gboolean vertical)
 	GtkWidget *viewport;
 	GtkWidget *group;
 	GtkWidget *button;
+	GtkWidget *arrow;
 
-	group = gtk_hbox_new (0, 0);
+	if (vertical)
+	{
+		group = gtk_vbox_new (0, 0);
+		g_object_set_data (G_OBJECT (group), "v", (gpointer)1);
+	} else
+		group = gtk_hbox_new (0, 0);
 	g_object_set_data (G_OBJECT (group), "c", callback);
 	gtk_widget_show (group);
 
 	viewport = gtk_viewport_new (0, 0);
 	gtk_viewport_set_shadow_type (GTK_VIEWPORT (viewport), GTK_SHADOW_NONE);
 	g_signal_connect (G_OBJECT (viewport), "size_request",
-							G_CALLBACK (tab_viewport_size_request), 0);
+							G_CALLBACK (tab_viewport_size_request), group);
 	gtk_box_pack_start (GTK_BOX (group), viewport, 1, 1, 0);
 	gtk_widget_show (viewport);
 
-	box = gtk_hbox_new (FALSE, 0);
+	if (vertical)
+		box = gtk_vbox_new (FALSE, 0);
+	else
+		box = gtk_hbox_new (FALSE, 0);
 	g_object_set_data (G_OBJECT (group), "i", box);
 	gtk_container_add (GTK_CONTAINER (viewport), box);
 	gtk_widget_show (box);
 
-	button = gtk_button_new_with_label (">");
+	button = gtk_button_new ();
+	arrow = gtk_arrow_new (vertical ? GTK_ARROW_DOWN : GTK_ARROW_RIGHT,
+								  GTK_SHADOW_NONE);
+	gtk_container_add (GTK_CONTAINER (button), arrow);
 	gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
 	g_signal_connect (G_OBJECT (button), "clicked",
-							G_CALLBACK (tab_scroll_right_clicked), group);
-	gtk_widget_show (button);
+							G_CALLBACK (tab_scroll_right_down_clicked), group);
 	gtk_box_pack_end (GTK_BOX (group), button, 0, 0, 0);
+	gtk_widget_show_all (button);
 
-	button = gtk_button_new_with_label ("<");
+	button = gtk_button_new ();
+	arrow = gtk_arrow_new (vertical ? GTK_ARROW_UP : GTK_ARROW_LEFT,
+								  GTK_SHADOW_NONE);
+	gtk_container_add (GTK_CONTAINER (button), arrow);
 	gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
 	g_signal_connect (G_OBJECT (button), "clicked",
-							G_CALLBACK (tab_scroll_left_clicked), group);
-	gtk_widget_show (button);
+							G_CALLBACK (tab_scroll_left_up_clicked), group);
 	gtk_box_pack_end (GTK_BOX (group), button, 0, 0, 0);
+	gtk_widget_show_all (button);
 
 	return group;
 }
@@ -211,9 +261,9 @@ tab_group_new (void *callback, gboolean vertical)
 /* traverse all the family boxes of tabs 
  *
  * A "group" is basically:
- * GtkHBox
+ * GtkV/HBox
  * `-GtkViewPort
- *   `-GtkHBox (inner box)
+ *   `-GtkV/HBox (inner box)
  *     `- GtkBox (family box)
  *        `- GtkToggleButton
  *        `- GtkToggleButton
@@ -249,7 +299,7 @@ tab_group_for_each_tab (GtkWidget *group,
 		{
 			child = tabs->data;
 
-			if (!GTK_IS_VSEPARATOR (child->widget))
+			if (!GTK_IS_SEPARATOR (child->widget))
 			{
 				if (callback (child->widget, i, usernum) != -1)
 					return i;
@@ -346,7 +396,7 @@ tab_focus (GtkWidget *tab)
 	tab_pressed_cb (GTK_TOGGLE_BUTTON (tab), group);
 }
 
-/* remove empty boxes and Vseparators */
+/* remove empty boxes and separators */
 
 void
 tab_group_cleanup (GtkWidget *group)
@@ -370,7 +420,7 @@ tab_group_cleanup (GtkWidget *group)
 		children = GTK_BOX (box)->children;
 		while (children)
 		{
-			if (!GTK_IS_VSEPARATOR (((GtkBoxChild *)children->data)->widget))
+			if (!GTK_IS_SEPARATOR (((GtkBoxChild *)children->data)->widget))
 			{
 				empty = FALSE;
 				break;
@@ -387,6 +437,38 @@ tab_group_cleanup (GtkWidget *group)
 	}
 }
 
+static void
+tab_add_sorted (GtkWidget *box, GtkWidget *tab)
+{
+#ifdef ALPHA_SORT
+	GList *list;
+	GtkBoxChild *child;
+	char *name = GTK_BUTTON (tab)->label_text;
+	int i = 0;
+
+	list = GTK_BOX (box)->children;
+	while (list)
+	{
+		child = list->data;
+		if (!GTK_IS_SEPARATOR (child->widget))
+		{
+			if (strcasecmp (GTK_BUTTON (child->widget)->label_text, name) > 0)
+			{
+				gtk_box_pack_start (GTK_BOX (box), tab, 0, 0, 0);
+				gtk_box_reorder_child (GTK_BOX (box), tab, i);
+				gtk_widget_show (tab);
+				return;
+			}
+		}
+		i++;
+		list = list->next;
+	}
+#endif
+
+	gtk_box_pack_start (GTK_BOX (box), tab, 0, 0, 0);
+	gtk_widget_show (tab);
+}
+
 static void
 tab_add_real (GtkWidget *group, GtkWidget *tab, void *family)
 {
@@ -405,8 +487,8 @@ tab_add_real (GtkWidget *group, GtkWidget *tab, void *family)
 
 		if (g_object_get_data (G_OBJECT (box), "f") == family)
 		{
-			gtk_box_pack_start (GTK_BOX (box), tab, 0, 0, 0);
-			gtk_widget_show (tab);
+			tab_add_sorted (box, tab);
+			gtk_widget_queue_resize (inner->parent);
 			return;
 		}
 
@@ -418,7 +500,7 @@ tab_add_real (GtkWidget *group, GtkWidget *tab, void *family)
 		children = GTK_BOX (box)->children;
 		while (children)
 		{
-			if (!GTK_IS_VSEPARATOR (((GtkBoxChild *)children->data)->widget))
+			if (!GTK_IS_SEPARATOR (((GtkBoxChild *)children->data)->widget))
 			{
 				empty = FALSE;
 				break;
@@ -436,19 +518,26 @@ tab_add_real (GtkWidget *group, GtkWidget *tab, void *family)
 	}
 
 	/* create a new family box */
-	box = gtk_hbox_new (FALSE, 0);
-#ifndef TABS_SPREAD
-	sep = gtk_vseparator_new ();
+	if (g_object_get_data (G_OBJECT (group), "v") != NULL)
+	{
+		/* vertical */
+		box = gtk_vbox_new (FALSE, 0);
+		sep = gtk_hseparator_new ();
+	} else
+	{
+		/* horiz */
+		box = gtk_hbox_new (FALSE, 0);
+		sep = gtk_vseparator_new ();
+	}
+
 	gtk_box_pack_end (GTK_BOX (box), sep, 0, 0, 4);
 	gtk_widget_show (sep);
 	gtk_box_pack_start (GTK_BOX (inner), box, 0, 0, 0);
-#else
-	gtk_container_add (GTK_CONTAINER (inner), box);
-#endif
 	g_object_set_data (G_OBJECT (box), "f", family);
 	gtk_box_pack_start (GTK_BOX (box), tab, 0, 0, 0);
 	gtk_widget_show (tab);
 	gtk_widget_show (box);
+	gtk_widget_queue_resize (inner->parent);
 }
 
 static void
@@ -484,13 +573,39 @@ tab_toggled_cb (GtkToggleButton *tab, gpointer user_data)
 	tab_release_cb (tab, NULL);
 }
 
+static char *
+truncate_tab_name (char *name, int max)
+{
+	char *buf;
+
+	if (max > 2 && g_utf8_strlen (name, -1) > max)
+	{
+		/* truncate long channel names */
+		buf = malloc (strlen (name) + 4);
+		strcpy (buf, name);
+		g_utf8_offset_to_pointer (buf, max)[0] = 0;
+		strcat (buf, "..");
+		return buf;
+	}
+
+	return NULL;
+}
+
 GtkWidget *
 tab_group_add (GtkWidget *group, char *name, void *family, void *userdata,
-			void *click_cb)
+			void *click_cb, int trunc_len)
 {
 	GtkWidget *but;
+	char *new_name;
 
-	but = gtk_toggle_button_new_with_label (name);
+	new_name = truncate_tab_name (name, trunc_len);
+	if (new_name)
+	{
+		but = gtk_toggle_button_new_with_label (new_name);
+		free (new_name);
+	} else
+		but = gtk_toggle_button_new_with_label (name);
+	gtk_widget_set_name (but, "xchat-tab");
 	/* used to trap right-clicks */
 	g_signal_connect (G_OBJECT (but), "button-press-event",
 						 	G_CALLBACK (click_cb), NULL);
@@ -523,9 +638,22 @@ tab_set_attrlist (GtkWidget *tab, PangoAttrList *list)
 }
 
 void
-tab_rename (GtkWidget *tab, char *new_name)
+tab_rename (GtkWidget *tab, char *name, int trunc_len)
 {
-	gtk_button_set_label (GTK_BUTTON (tab), new_name);
+	PangoAttrList *attr = gtk_label_get_attributes (GTK_LABEL (GTK_BIN (tab)->child));
+	char *new_name;
+
+	new_name = truncate_tab_name (name, trunc_len);
+	if (new_name)
+	{
+		gtk_button_set_label (GTK_BUTTON (tab), new_name);
+		free (new_name);
+	} else
+		gtk_button_set_label (GTK_BUTTON (tab), name);
+	gtk_widget_queue_resize (tab->parent->parent->parent);
+
+	if (attr)
+		gtk_label_set_attributes (GTK_LABEL (GTK_BIN (tab)->child), attr);
 }
 
 void
@@ -565,3 +693,69 @@ tab_remove (GtkWidget *tab)
 		tab_group_switch (group, 0, FALSE);
 	}
 }
+
+GtkOrientation
+tab_group_get_orientation (GtkWidget *group)
+{
+	int vertical;
+	vertical = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (group), "v"));
+	return (vertical ? GTK_ORIENTATION_VERTICAL : GTK_ORIENTATION_HORIZONTAL);
+}
+
+GtkWidget *
+tab_group_set_orientation (GtkWidget *group, gboolean vertical)
+{
+	GtkWidget *box;
+	GtkWidget *new_group;
+	GList *boxes;
+	int is_vertical;
+
+	is_vertical = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (group), "v"));
+	if ((vertical && is_vertical) || (!vertical && !is_vertical))
+		return group;
+
+	new_group = tab_group_new (g_object_get_data (G_OBJECT (group), "c"),
+								vertical);
+	g_object_set_data (G_OBJECT (new_group), "foc",
+						g_object_get_data (G_OBJECT (group), "foc"));
+	box = g_object_get_data (G_OBJECT (group), "i");
+	boxes = GTK_BOX (box)->children;
+	while (boxes)
+	{
+		GtkWidget *family_box;
+		GList *children;
+
+		family_box = ((GtkBoxChild *) boxes->data)->widget;
+		children = GTK_BOX (family_box)->children;
+
+		while (children)
+		{
+			GtkWidget *child;
+
+			child = ((GtkBoxChild *) children->data)->widget;
+
+			if (GTK_IS_TOGGLE_BUTTON (child))
+			{
+				void *family;
+
+				gtk_widget_ref (child);
+				gtk_container_remove (GTK_CONTAINER (family_box), child);
+				g_signal_handlers_disconnect_by_func (G_OBJECT (child),
+									G_CALLBACK (tab_pressed_cb), group);
+				g_signal_connect (G_OBJECT (child), "pressed",
+									G_CALLBACK (tab_pressed_cb), new_group);
+				family = g_object_get_data (G_OBJECT (child), "f");
+				g_object_set_data (G_OBJECT (child), "g", new_group);
+				tab_add_real (new_group, child, family);
+				gtk_widget_unref (child);
+				children = GTK_BOX (family_box)->children;
+			} else
+				children = children->next;
+		}
+
+		boxes = boxes->next;
+	}
+
+	return new_group;
+}
+
